/*
	File:		memstat.c
	Programmer:	David Cook / cookware
	Phone:		(317) 769-5049
	E-Mail:		iceman@cerf.net
			dcook@npal.rn.com


	Disclaimer:	While this program is fairly well debugged, neither 
			cookware nor its employees assume any responsibility 
			for errors in this code or any damage caused by running
			this program.

	About:		cookware is an independent contracting company composed
			of (currently) two very hard working individuals who,
			if you like this program and the quality of the work -
			and have work you need done of your own, would love to
			hear from you!

	Tabs:		This file was written with default tab stops - every
			8 characters.  To display the file in its best form,
			use these settings.

	Description:	Memstat is a stand-alone program which displays a 
			'chart' on your screen.  The chart is divided into 
			three basic areas (besides the copyright).  Each area 
			monitors a different segment of the machine, as follows:

			* SWAP - monitors your available swap space
			* PHYS - monitors your available physical memory space
			* DISK - monitors available space on your current drive

			This program was written to allow developers and users 
			a method to monitor these three areas, while programs 
			are running, to determine if a program is using more 
			memory than the resources have available.

	Status:		This program is in the public domain, and is free to
			dispense world wide on Silicon Graphics computer
			systems.  I only request that you please leave the
			copyright messages untouched.  If you make significant
			additions to this code, and wish to use your own logo
			or copyright, please ADD it to the existing one.  Thank
			you very much.

	Options:	memstat may be invoked standalone, or with the following
			options:

			  usage:	memstat [-adfhkpstv]

			-a        Turns on the 'alarm'.  This option is used 
				  in conjunction with -d, -p and/or -s.  When 
				  used with these options, the alarm option 
				  will beep the BELL each time the individual
		    		  limits are exceeded.  Additionally, a message
				  is printed on 'stderr' explaining the alarm 
				  and the individual alarm slider is displayed 
				  in purple instead of green.

			-d n      Turns on 'alarm watch' mode for the DISK 
				  display.  Supply a value for 'n' between 0 
				  and 100.  The value you supply indicates the 
				  PERCENTAGE of disk AVAILABLE that you want
		    		  to monitor for. If the current disk available
				  slips BELOW the value you specify for 'n', 
				  the bar will turn purple.  If you specified 
				  the -a option, an alarm will also be sounded.

			-f path   memstat defaults the DISK display for the file
				  system that you were in when you launched 
				  memstat.  If you wish to override the DISK 
				  display, to point to a different file system,
				  simply specify the -f option, followed by the
		    		  full path of ANY readable file which exists 
				  on the desired filesystem.  Note that if the 
				  file you specify does not exist, memstat will
				  display a message to that effect and will 
				  monitor for the existance of the file.  Once 
				  the file exists, the stats for that drive 
				  will be displayed.

		    		  If the file is removed during the display, 
				  the display will again wait for the file to 
				  reappear.  This is very useful for debugging 
				  large files which may come and go at various
				  times.

			-h	  Displays a help message with info on command 
				  line options.

			-k pname  This option will KILL a process when any of 
				  the alarms (eg., -d, -p or -s) activate.  
			          memstat sends a 

	                                 killall -KILL 'pname'

		    		  to the specified process.  'pname' should be 
				  the process NAME, NOT its process ID number.
				  Note that depending on the process, memstat 
				  might need to be super-user to do this.

			-p n      Turns on 'alarm watch' mode for the PHYS 
				  display.  Supply a value for 'n' between 0 
				  and 100.  The value you supply indicates the 
				  PERCENTAGE of physical memory AVAILABLE that 
		    		  you want to monitor for.  If the current 
				  physical memory available slips BELOW the 
				  value you specify for 'n', the bar will turn 
				  purple.  If you specified the -a option, an 
	            		  alarm will also be sounded.

			-s n      Turns on 'alarm watch' mode for the SWAP 
				  display.  Supply a value for 'n' between 0 
				  and 100.  The value you supply indicates the 
				  PERCENTAGE of swap space AVAILABLE that you 
				  want to monitor for.  If the current swap 
				  space available slips BELOW the value you 
				  specify for 'n', the bar will turn purple.  
				  If you specified the -a option, an alarm will
				  also be sounded.

			-t n	  Sets the frequency for update to 'n' seconds.
				  Note that 'n' defaults to 1. If the -t option
				  is set to zero, the timer is disabled and 
				  polling of memory status is set to full speed
				  - constant display.

			-v	  Turns on 'verbose' mode.  Under verbose mode,
				  anytime the bars change (only bar movement, 
				  not micromovement) the current values are 
				  printed to 'stderr'.  Additionally, if -d, -p
				  -s and/or -a is used, alarm information is 
				  also printed when alarms occur.

	Examples:	memstat in its default mode updates the display each
			second.  No alarms are used and Verbose is set to OFF.
			The drive monitored is the drive you were on when the
			program was run.  The display appears at the bottom of 
			the screen.

				memstat

			To launch memstat with updates every 5 seconds, use:

				memstat -t 5

			To launch memstat with constant updates, use:

				memstat -t 0

			To launch memstat such that it checks constantly (-t 0),
			issues an alarm when DISK is below 25%, PHYSICAL is
			below 10% and SWAP is below 50%, and monitors whether
			a core file has been created in a particular directory:

				memstat -t 0 -d 25 -p 10 -s 50 -f /tmp/core

			To launch memstat such that it will kill a process named
			'paint' if disk memory goes below 10%:

				memstat -d 10 -k paint


	For Hackers:	This program utilizes the following 'interesting' SGI
			and UNIX concepts:

				* signals      - For timing the poll cycle
				* SGI graphics - For displaying the chart
				* statfs       - For aquiring disk information 
				* sysmips      - For swap info
				* sysmp        - For physical memory info
				* getopt       - For parsing the command line

			Routines found in this file - and what they do:

			alarm_catch          - Routine that updates display
			ge_draw_basic_button - Draws a bar (button) background
			ip_font              - Handles text display
			show_disk            - Displays disk bar background
			show_help            - Routine that displays help text
			show_logo            - Displays copyright bar
			show_phys            - Displays physical bar background
			show_swap            - Displays swap bar background
			update_disk          - Updates the disk bar
			update_phys          - Updates the physical bar
			update_swap          - Updates the swap bar

*/

/***************************  INCLUDE STATEMENTS  *****************************/
#include	<signal.h>
#include	<stdio.h>
#include	<unistd.h>
#include	<gl/device.h>
#include	<gl/gl.h>
#include	<sys/immu.h>
#include	<sys/time.h>
#include	<sys/statfs.h>
#include	<sys/swap.h>
#include	<sys/sysmips.h>
#include	<sys/sysmp.h>

/*******************************  PROTOTYPES  *********************************/
void		ip_font(void);
void		show_phys(void);
void		show_disk(void);
void		show_swap(void);
void		update_phys(int,int);
void		update_disk(int,int);
void		update_swap(int,int);
void		show_logo(void);
void		show_help(void);
void		alarm_catch(int,int,struct sigcontext);
void		ge_draw_basic_button(int,int,int,int);

/********************************  EXTERNALS  *********************************/
extern char	*optarg;			/* System defines this */
extern int	optind;				/* System defines this */

/*******************************  DEFINITIONS  ********************************/
#define		ptok(x)		((x)<<12)
#define		BAR		245
#define		DARK		0
#define		BUTTON		305
#define		BARCOLOR	0x3f6f00
#define		HIGHCOLOR	0x006f6f
#define		BASIC		0x6f6f6f
#define		ALERT		0x6f003f
#define		BARHEIGHT	22

/*******************************  GLOBAL DATA  ********************************/
int	window_id;				/* Holds memstats window id */
int	freeswap		= 0;		/* Holds free swap space */
int	totalswap		= 0;		/* Holds total swap space */
int	lastswap		= 0;		/* Last swap value */
int	lastphys		= 0;		/* Last phys value */
int	lastdisk		= 0;		/* Last disk value */
char	*cd;					/* Current path goes here */
char	f_filename[256];			/* Used to hold path for -f */
char	*process_name		= 0;		/* Used to hold pname for -k */
char	swapfix			= 0;		/* Flag to redisplay swap */
char	physfix			= 0;		/* Flag to redisplay phys */
char	diskfix			= 0;		/* Flag to redisplay disk */
char	alert			= FALSE;	/* TRUE if -a is used */
char	dalert			= FALSE;	/* TRUE if -d is used */
char	kalert			= FALSE;	/* TRUE if -k is used */
char	palert			= FALSE;	/* TRUE if -p is used */
char	salert			= FALSE;	/* TRUE if -s is used */
char	verbose			= FALSE;	/* TRUE if -v is used */
int	spercent		= 0;		/* Set to % for swap alarm */
int	ppercent		= 0;		/* Set to % for phys alarm */
int	dpercent		= 0;		/* Set to % for disk alarm */
int	button_color		= BASIC;	/* Color for the buttons */
int	dflag			= FALSE;	/* TRUE if disk needs refresh */
int	seconds			= 1;		/* Timer value */
int	screen_x_size		= 0;		/* Used to hold screen width  */
int	screen_y_size		= 0;		/* Used to hold screen height */
struct	itimerval		timer1,timer2;	/* Used by the timers */

/*******************************  SOURCE CODE  ********************************/

/* *****************************************************************************

	Routine:	main()
	Author:		David Cook

	Description:	This is the main for the Memory Visulizer

***************************************************************************** */
main(int argc,char *argv[])
{
int	d1,c;
short	d2;

	/*  Get the current working directory (for 'statfs' ) */
	cd = getcwd((char *)NULL,80);

	/*  Parse the command line looking for the users options  */
	if (argc > 1) {
          while ((c = getopt(argc,argv,"abd:f:hk:p:s:t:v")) != -1) {
            switch (c) {

              case 'a':		/* '-a' ALARM option */
	        alert = TRUE;
              break;

              case 'd':		/* '-d n' Set disk alarm percentage */
	        sscanf(optarg,"%d",&dpercent);
	        if ((dpercent < 0) || (dpercent > 100)) {
	          fprintf(stderr,"memstat:  Error in -d %s.  '%s' must be >=0 and <= 100\n",optarg,optarg);
	          exit(-1);
	        }
	        dalert = TRUE;
              break;

              case 'f':		/* '-f path' Set DISK path and file monitor */
	        sscanf(optarg,"%s",(char *)f_filename);
	        cd = f_filename;
              break;

              case 'h':		/* '-h' Show the user the standard help line */
	        show_help();
	        exit(0);
              break;

	      case 'k':		/* '-k pname' Sets process to kill upon alarm */
	        kalert = TRUE;
	        process_name = (char *)strdup(optarg);
	      break;

              case 'p':
	        sscanf(optarg,"%d",&ppercent);
	        if ((ppercent < 0) || (ppercent > 100)) {
	          fprintf(stderr,"memstat:  Error in -p %s.  '%s' must be >=0 and <= 100\n",optarg,optarg);
	          exit(-1);
	        }
	        palert = TRUE;
              break;

              case 's':		/* '-s n' Sets SWAP alarm percentage */
	        sscanf(optarg,"%d",&spercent);
	        if ((spercent < 0) || (spercent > 100)) {
	          fprintf(stderr,"memstat:  Error in -s %s.  '%s' must be >=0 and <= 100\n",optarg,optarg);
	          exit(-1);
	        }
	        salert = TRUE;
              break;

              case 't':		/* '-t n' Sets number of seconds for update */
	        sscanf(optarg,"%d",&seconds);
              break;

              case 'v':		/* '-v' Sets VERBOSE mode */
	        verbose = TRUE;
              break;

	      default:		/* Whoops, give 'em the help message and exit */
	        show_help();
	        exit(-1);
	      break;
	    }
	  }
        }

	/*  Determine the X and Y size of the screen  */
	screen_x_size = getgdesc(GD_XPMAX);
	screen_y_size = getgdesc(GD_YPMAX);

	/*  Open the window that contains the memstat display  */
	noborder();
	prefposition(0,screen_x_size-1,0,BARHEIGHT-1);
	winopen("memstat");
	ortho2(0.0,((float)screen_x_size),0.0,BARHEIGHT.0);
	viewport(0,screen_x_size-1,0,BARHEIGHT-1);
	RGBmode();
	gconfig();
	drawmode(NORMALDRAW);
	cpack(DARK);
	clear();
	window_id = winget();

	/* Queue the appropriate EVENTs that are not automatically defaulted */
	qdevice(WINQUIT);

	/* Issue our greeting to the user */
	fprintf(stderr,"\nmemstat - Version 1.4 - (c) 1992 cookware\n(use 'memstat -h' for help with options)\n\n");

	/* Engage the raster font and do the initial display of the chart */
	ip_font();
	show_logo();
	show_swap();
	show_phys();
	show_disk();

	/*  Set the signal for an alarm to interrupt and poll  */
	if (seconds > 0) {
	  signal(SIGALRM,(void(*)(int,int,struct sigcontext))alarm_catch);
	  timer1.it_value.tv_sec    = seconds;
	  timer1.it_value.tv_usec   = 0;
	  timer1.it_interval.tv_sec = 0;
	  if (setitimer(ITIMER_REAL,&timer1,&timer2) != 0) { 
	    perror("setitimer");
	    exit(-1);
	  }
	}

	/***************** MAIN EVENT LOOP ****************/
	/*  Handle the queue  */
	while (TRUE) {
	  /*  If -t 0 was specified, handle it seperatly  */
	  if (seconds == 0) {
	    update_swap(0,0);
	    update_phys(BUTTON,0);
	    update_disk(BUTTON+BUTTON,0);
	  }
	  /*  Now, determine if anything has happened in the queue  */
	  if (qtest()) {
	    d1 = qread(&d2);
	    switch (d1) {
	      case WINQUIT:		/* Our process has been killed */
	      case WINSHUT:		/* Our process has been killed */
	        gexit();
	        exit(0);
	      break;

	      case INPUTCHANGE:		/* Our focus has been changed */
	        winset(d2);
	      break;

	      case REDRAW:		/* Some window has been changed */
	        if (d2 == window_id) {
	          /* It's OUR window - disable the timer for a second or two */
		  if (seconds > 0) sighold(SIGALRM);

		  /* Redraw everything */
	          winset(d2);
	          cpack(DARK);
	          rectfi(0,0,screen_x_size,BARHEIGHT);
	          physfix = TRUE;
	          diskfix = TRUE;
	          swapfix = TRUE;
	          show_logo();
	          show_swap();
	          show_phys();
	          show_disk();

	          /* Turn the timer back on */
		  if (seconds > 0) sigrelse(SIGALRM);
	        }
	      break;
	    }
	  }
	}
}

/* *****************************************************************************

	Routine:	alarm_catch()
	Programmer:	David Cook - cookware

	Description:	This routine will handle the alarm signal when it is 
			issued by the system.  Each time the alarm is issued,
			the bars are redisplayed and the signal is reengaged.

	Syntax:		alarm_catch(a,b,c);

	Where:		a = (int)		- See the 'signal' command
			b = (int)		- See the 'signal' command
			c = (struct sigcontext)	- See the 'signal' command

	Notes:		(1)  The values input to this routine must be there
			     for the system, eg., define 'em as I did in the
			     declaration below... but I don't use 'me - don't
			     need to.  But because the system is calling me,
			     I gotta put 'em in.  FYI

***************************************************************************** */
void	alarm_catch(int a,int b,struct sigcontext c)
{
	if (seconds > 0) {
	  /*  Show the bars  */
	  update_swap(0,0);
	  update_phys(BUTTON,0);
	  update_disk(BUTTON+BUTTON,0);

	  /*  Reset the alarm  */
	  signal(SIGALRM,(void(*)(int,int,struct sigcontext))alarm_catch);
	  if (setitimer(ITIMER_REAL,&timer1,&timer2) != 0) { 
	    perror("setitimer");
	    exit(-1);
	  }
	}

	return;
}

/* *****************************************************************************

	Routine:	show_swap()
	Programmer:	David Cook - cookware

	Description:	This routine will display the SWAP bar, aint with its
			contents.  This is only called upon initial startup
			and anytime the window is moved or unhidden.

	Syntax:		show_swap();

***************************************************************************** */
void show_swap()
{
	/*  Draw the slider 'shell'  */
        ge_draw_basic_button(0,0,BUTTON,BARHEIGHT);

	/*  Show the text  */
	cpack(0x252525);
	cmov2i(5,7);
	charstr("swap");

	/*  Show the slider itself  */
	update_swap(0,0);

	return;
}

/* *****************************************************************************

	Routine:	show_phys()
	Programmer:	David Cook - cookware

	Description:	This routine will display the CORE bar, aint with its
			contents.  This is only called upon initial startup
			and anytime the window is moved or unhidden.

	Syntax:		show_phys();

***************************************************************************** */
void show_phys()
{
	/*  Draw the slider 'shell'  */
        ge_draw_basic_button(BUTTON,0,BUTTON,BARHEIGHT);

	/*  Show the text  */
	cpack(0x252525);
	cmov2i(BUTTON+5,7);
	charstr("phys");

	/*  Show the slider itself  */
	update_phys(BUTTON,0);

	return;
}

/* *****************************************************************************

	Routine:	show_disk()
	Programmer:	David Cook - cookware

	Description:	This routine will display the DISK bar, aint with its
			contents.  This is only called upon initial startup
			and anytime the window is moved or unhidden.

	Syntax:		show_disk();

***************************************************************************** */
void show_disk()
{
	/*  Draw the slider 'shell'  */
        ge_draw_basic_button(BUTTON+BUTTON,0,BUTTON,BARHEIGHT);

	/*  Show the slider itself  */
	update_disk(BUTTON+BUTTON,0);

	return;
}

/* *****************************************************************************

	Routine:	show_logo()
	Programmer:	David Cook - cookware

	Description:	This routine will display the cookware name and the
			copyright logo, to fill out the bar to the right of the
			screen.

	Syntax:		show_logo();

***************************************************************************** */
void show_logo()
{
int	x;

	/*  Generate the button  */
	x = BUTTON+BUTTON+BUTTON;
        ge_draw_basic_button(x,0,screen_x_size-x,BARHEIGHT);

	/*  Generate the text  */
	cpack(0x0);
	cmov2i(x+5,6);
	charstr("\1");
	cpack(0x252525);
	cmov2i(x+23,7);
	charstr("cookware");

	return;
}

/* *****************************************************************************

	Routine:	ge_draw_basic_button    << ge Services >>
	Programmer:	David Cook - cookware

	Copyright:	This routine belongs to the Interplay menu manager
			system.  It was removed and slightly modified to
			allow memstat to be distributed without out the
			overhead of Interplay.  Note that the look and feel
			of this routine are the copyright of cookware, and
			are part of the Interplay menu manager.  For more
			information about Interplay, contact David Cook at
			(317) 769-5049

	Description:	This routine will draw a basic button, with no text.

        Syntax:		ge_draw_basic_button(llx,lly,width,height);

	Where:		llx    - (int) Relative LL X of the button
			lly    - (int) Relative LL Y of the button
			width  - (int) Width of the button
			height - (int) Height of the button

***************************************************************************** */
void	ge_draw_basic_button(int llx,int lly,int width,int height)
{
long	a[6][2];

	concave(TRUE);

	/*  Create the button polygon data  */
	a[4][0] = a[0][0] = a[3][0] = (long)llx;
	a[5][0] = a[1][0] = a[2][0] = (long)(llx + width - 1);
	a[0][1] = a[1][1] =           (long)lly;
	a[2][1] = a[3][1] =           (long)(lly + height - 1);
	a[5][1] = a[4][1] =           (long)(lly + (height / 2));

	/*  Draw the button  */
	bgnpolygon();
	  cpack(DARK);
	  v2i(a[0]);
	  cpack(button_color);
	  v2i(a[4]);
	  v2i(a[5]);
	  cpack(DARK);
	  v2i(a[1]);
	endpolygon();

	bgnpolygon();
	  cpack(button_color);
	  v2i(a[4]);
	  cpack(DARK);
	  v2i(a[3]);
	  v2i(a[2]);
	  cpack(button_color);
	  v2i(a[5]);
	endpolygon();

	return;
}

/* *****************************************************************************

	Routine:	update_swap()
	Programmer:	David Cook - cookware

	Description:	This routine will display the CONTENTS of the SWAP bar
			only (eg., the SLIDER portion only!).

	Syntax:		update_swap(x,y);

	Where:		x = (int) The lower left X location of the bar.
			y = (int) The lower left Y location of the bar.

***************************************************************************** */
void update_swap(int x,int y)
{
register swpt_t	*st;
register swpi_t	*si;
swpt_t		swapt[MSFILES];
swpi_t		swpi;
int		val,z;
char		b[80];

	/*  Get the swap information (courtesy SGI 4Dgifts!)  */
	totalswap = freeswap = 0;
	st         = swapt; 
	si         = &swpi; 
	si->si_cmd = SI_LIST; 
	si->si_buf = (char *) st;
	if (sysmips(SMIPSSWPI, si) < 0) { 
	  perror("swap"); 
	  return;
	}
	for (val = 0; val < MSFILES;  val++, st++) { 
	  if (st->st_ucnt == NULL) continue;
	  totalswap += st->st_npgs  * NBPP;
	  freeswap  += st->st_nfpgs * NBPP;
	}

	/*  Display the 'Slider'  */
	val = (int)((((float)freeswap) / ((float)totalswap)) * BAR.0);
	if ((lastswap != val) || swapfix) {
	  /*  Show the black outline for good looks  */
	  cpack(DARK);
	  rectfi(x+51,y+9,x+BAR+53,y+13);

	  /*  Make the total inner portion red  */
	  cpack(0x3f006f);
	  rectfi(x+52,y+10,x+BAR+52,y+12);

	  /*  Determine the color we need  */
	  if (seconds == 0) cpack(BARCOLOR);
	  else              cpack((lastswap != val) ? HIGHCOLOR : BARCOLOR);

	  /*  If we are checking for a SWAP alert condition...  */
	  if (salert) {
	    if((z=(int)(100.0*((float)freeswap)/((float)totalswap)))<=spercent){
	      /* WE HAVE AN ALERT!!! */
	      cpack(ALERT);
 	      if (verbose || alert) 
	        fprintf(stderr,"memstat:  [SWAP ALERT]  Available Space at %d%%\n",z);
	      if (alert) ringbell();

	      /*  Check for -k option  */
	      if (kalert) {
	        sprintf(b,"killall -KILL %s",process_name);
	        system(b);
	        fprintf(stderr,"memstat:  [SWAP ALERT]  Process %s killed upon request - space at %d\n",b,z);
	      }
	    }
	  }

	  /*  Show the green portion (or purple or yellow as the case may be) */
	  rectfi(x+52+(BAR-val),y+10,x+52+val+(BAR-val),y+12);

	  /*  Set whether or not we need to redisplay next time  */
	  swapfix = (lastswap != val) ? 1 : 0;
	}

	if ((lastswap != val) && verbose)
	  fprintf(stderr,"memstat:  [SWAP]  %d bytes swap available out of %d\n",
	          freeswap,totalswap);
	lastswap = val;

	return;
}

/* *****************************************************************************

	Routine:	update_phys()
	Programmer:	David Cook - cookware

	Description:	This routine will display the CONTENTS of the PHYSICAL
			bar only (eg., the SLIDER portion only!).

	Syntax:		update_phys(x,y);

	Where:		x = (int) The lower left X location of the bar.
			y = (int) The lower left Y location of the bar.

***************************************************************************** */
void update_phys(int x,int y)
{
int		m_info,pm,fm,val,z;
struct	rminfo	r;
char	b[80];
        
	/*  Extract the information from the system  */
	m_info = sysmp(MP_SASZ, MPSA_RMINFO);
	sysmp(MP_SAGET, MPSA_RMINFO, (char *) &r, m_info);
	pm = ptok(r.physmem);
	fm = ptok(r.freemem);

	/*  Display the 'Slider'  */
	val = (int)((((float)fm) / ((float)pm)) * BAR.0);
	if ((lastphys != val) || physfix) {
	  /*  Show the black outline for good looks  */
	  cpack(DARK);
	  rectfi(x+51,y+9,x+BAR+53,y+13);

	  /*  Make the total inner portion red  */
	  cpack(0x3f006f);
	  rectfi(x+52,y+10,x+BAR+52,y+12);

	  /*  Determine the color we need  */
	  if (seconds == 0) cpack(BARCOLOR);
	  else              cpack((lastphys != val) ? HIGHCOLOR : BARCOLOR);

	  /*  If we are checking for a PHYS alert condition...  */
	  if (palert) {
	    if((z=(int)(100.0*((float)fm)/((float)pm))) <= ppercent) {
	      /* WE HAVE AN ALERT!!! */
	      cpack(ALERT);
 	      if (verbose || alert) 
	        fprintf(stderr,"memstat:  [PHYS ALERT]  Available Space at %d%%\n",z);
	      if (alert) ringbell();

	      /*  Check for -k option  */
	      if (kalert) {
	        sprintf(b,"killall -KILL %s",process_name);
	        system(b);
	        fprintf(stderr,"memstat:  [PHYS ALERT]  Process %s killed upon request - space at %d\n",b,z);
	      }
	    }
	  }

	  /*  Show the green portion (or purple or yellow as the case may be) */
	  rectfi(x+52+(BAR-val),y+10,x+52+val+(BAR-val),y+12);

	  /*  Set whether or not we need to redisplay next time  */
	  physfix = (lastphys != val) ? 1 : 0;
	}

	if ((lastphys != val) && verbose)
	  fprintf(stderr,"memstat:  [PHYS]  %d bytes physical memory available out of %d\n",
	          fm,pm);
	lastphys = val;

	return;
}

/* *****************************************************************************

	Routine:	update_disk()
	Programmer:	David Cook - cookware

	Description:	This routine will display the CONTENTS of the DISK
			bar only (eg., the SLIDER portion only!).  If the
			specified file (eg., the default directory or the
			path specified by the -f option) does not exist, the
			bar is replaced with text tellint the user that the
			system is waiting for the file to exist (the wait
			occurs interactivly - eg., it does NOT cause the
			other activities to cease).

	Syntax:		update_disk(x,y);

	Where:		x = (int) The lower left X location of the bar.
			y = (int) The lower left Y location of the bar.

***************************************************************************** */
void update_disk(int x,int y)
{
struct	statfs	buf;
int		val,totalb,freeb,z,i,l;
char		b[80];
char		c[80];
        
	/*  Check to see of the path (cd) exists, if so, get statistics  */
	if (statfs(cd,&buf,sizeof(struct statfs),0) == 0) {
	  /*  Check to see if the disk button background needs refreshing  */
	  if (dflag!=0) ge_draw_basic_button(BUTTON+BUTTON,0,BUTTON,BARHEIGHT);

	  /*  Calculate the total bytes and free bytes  */
	  totalb = buf.f_bsize * buf.f_blocks;
	  freeb  = buf.f_bsize * buf.f_bfree;

	  /*  Display the 'Slider'  */
	  val = (int)((((float)freeb) / ((float)totalb)) * BAR.0);
	  if ((lastdisk != val) || diskfix) {

	    /*  Show the black outline for good looks  */
	    cpack(DARK);
	    rectfi(x+51,y+9,x+BAR+53,y+13);

	    /*  Make the total inner portion red  */
	    cpack(0x3f006f);
	    rectfi(x+52,y+10,x+BAR+52,y+12);

	    /*  Determine the color we need  */
	    if (seconds == 0) cpack(BARCOLOR);
	    else              cpack((lastdisk != val) ? HIGHCOLOR : BARCOLOR);

	    /*  If we are checking for a DISK alert condition...  */
	    if (dalert) {
	      if ((z=(int)(100.0*((float)freeb)/((float)totalb))) <= dpercent) {
	        /* WE HAVE AN ALERT!!! */
	        cpack(ALERT);
 	        if (verbose || alert) 
	          fprintf(stderr,"memstat:  [DISK ALERT]  Available Space at %d%%\n",z);
	        if (alert) ringbell();
	        /*  Check for -k option  */
	        if (kalert) {
	          sprintf(c,"killall -KILL %s",process_name);
	          system(c);
	          fprintf(stderr,"memstat:  [DISK ALERT]  Process %s killed upon request - space at %d\n",c,z);
	        }
	      }
	    }
	    /*  Show the green portion (or purple or yellow as it may be) */
	    rectfi(x+52+(BAR-val),y+10,x+52+val+(BAR-val),y+12);

	    /*  Display the text  */
	    cpack(0x252525);
	    cmov2i(x+5,y+7);
	    charstr("disk");

	    /*  Set whether or not we need to redisplay next time  */
	    diskfix = (lastdisk != val) ? 1 : 0;
	  }

	  if ((lastdisk != val) && verbose)
	    fprintf(stderr,
	            "memstat:  [DISK]  %d bytes disk available out of %d\n",
	            freeb,totalb);
	  lastdisk = val;
	  dflag = 0;

	} else {
	  /*  Whoops, the file has disappeared on us, or maybe was not there
	      to begin with... better display a different message
	  */
	  if (dflag == 0) {
	    fprintf(stderr,"memstat:  Will provide DISK display when '%s' exists\n",cd);
	    /*  Darw the button  */
            ge_draw_basic_button(BUTTON+BUTTON,0,BUTTON,BARHEIGHT);

	    /*  Put the text up  */
	    cpack(0x252525);
	    cmov2i(x+5,y+7);
	    l = strlen(cd);
	    for (i = l-1; i >= 0; i--) if (cd[i] == '/') break;
	    l = (i <= 0) ? 0 : (i+1);
	    sprintf(b,"* %s * does not exist yet",&cd[l]);
	    charstr(b);

	    /*  Set it to redisplay next times  */
	    dflag   = TRUE;
	    diskfix = TRUE;
	  }
	}

	return;
}

/* *****************************************************************************

	Routine:	show_help()
	Author:		David Cook

	Description:	This routine will display the help information

***************************************************************************** */
void show_help()
{
	fprintf(stderr,"\n\n    -----------------------------------------------------------------------\n");
	fprintf(stderr,"    :      memstat - Version 1.4 - (c) 1992 cookware (317) 769-5049       :\n");
	fprintf(stderr,"    :                                                                     :\n");
	fprintf(stderr,"    : A utility for monitoring Swap space, Physical memory and Disk space :\n");
	fprintf(stderr,"    -----------------------------------------------------------------------\n\n");
	fprintf(stderr,"usages:   memstat [-adfhpstv]\n\n");
	fprintf(stderr,"     -a       Enable audible alarm for -d, -p or -s options\n");
	fprintf(stderr,"     -d n     Set alarm level for DISK display at 'n' percent (0-100)\n");
	fprintf(stderr,"     -f path  Sets DISK display to show drive associated with 'path'\n");
	fprintf(stderr,"     -h       Displays this help screen\n");
	fprintf(stderr,"     -k pname Kills process 'pname' (signal -9) when -d, -p or -s alarm occurs\n");
	fprintf(stderr,"     -p n     Set watch level for PHYSICAL display at 'n' percent (0-100)\n");
	fprintf(stderr,"     -s n     Set watch level for SWAP display at 'n' percent (0-100)\n");
	fprintf(stderr,"     -t n     Set refresh frequency of 'n' seconds (default is 1)\n");
	fprintf(stderr,"     -v       Turns VERBOSE mode on. Sends info to 'stderr' as it happens\n\n");
	fprintf(stderr,"Notes:  * memstat displays three bars which indicate (left to right) available\n");
	fprintf(stderr,"	  SWAP space, available PHYSical memory and available DISK space.\n\n");
	fprintf(stderr,"	* The green portion indicats WHAT IS AVAILABLE FOR USE.\n");
	fprintf(stderr,"	  The red portion shows WHAT IS IN USE.  When a bar is updated, it\n");
	fprintf(stderr,"	  shows yellow, instead of green, for one update, so you can identify\n");
	fprintf(stderr,"	  the change (this does not occur if you used -t 0).\n\n");
	fprintf(stderr,"	* The green will become deep purple when the percentage of available\n");
	fprintf(stderr,"	  space drops below the values you set via -d, -p and -s, indicating\n");
	fprintf(stderr,"	  that the value is not below the alarm setting.\n\n");
	fprintf(stderr,"	* Use the -f option to display stats for a different drive other than\n");
	fprintf(stderr,"	  the one you were on when you ran memstat.  'path' should point to\n");
	fprintf(stderr,"	  some readable file on that drive.  The -f option may also be used\n");
	fprintf(stderr,"	  to show existance or removal of a file.\n\n");
	fprintf(stderr,"	* Use -t 0 to cause memstat to poll the system constantly, and -t n\n");
	fprintf(stderr,"	  to cause memstat to poll at 'n' second intervals\n\n");
	return;
}

/****************************  RASTER FONT DATA  ******************************/
/*  The following raster font data enables the font and other symbols used in
    the memstat display.  Note that the data and code appearing below is part
    of the Interplay menu manager system, and thus is under copyright of
    cookware.
*/

unsigned short	raster_f[]={
	0x1f80,0x2040,0x4020,0x8f10,0x9010,0x9010,0x9010,0x9010,0x8f10,0x4020,
	0x2040,0x1f80,0x0000,0x0000,0x0000,0xc000,0xc000,0x0000,0x8000,0x8000,
	0x8000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0x0000,0x0000,0x0000,
	0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x4800,0xd800,0xd800,
	0xd800,0x6c00,0x6c00,0x6c00,0xff00,0xff00,0x3600,0x3600,0x7f80,0x7f80,
	0x1b00,0x1b00,0x1b00,0x1000,0x7c00,0xfe00,0xd600,0x1600,0x1e00,0x3c00,
	0x7800,0xf000,0xd000,0xd600,0xfe00,0x7c00,0x1000,0x1060,0x08f0,0x0998,
	0x0598,0x04f0,0x0260,0x3200,0x7900,0xcd00,0xcc80,0x7880,0x3040,0x39c0,
	0x7f80,0xc700,0xc700,0xcf80,0x7dc0,0x38c0,0x3c00,0x6600,0x6600,0x7e00,
	0x3c00,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
	0x0000,0x8000,0x4000,0xc000,0xc000,0x3000,0x7000,0x6000,0xc000,0xc000,
	0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0x6000,0x7000,0x3000,
	0xc000,0xe000,0x6000,0x3000,0x3000,0x3000,0x3000,0x3000,0x3000,0x3000,
	0x3000,0x3000,0x6000,0xe000,0xc000,0x0000,0x0000,0x0000,0x0000,0x0000,
	0x8800,0x7000,0x7000,0xa800,0x2000,0x0000,0x0000,0x0000,0x1800,0x1800,
	0x1800,0xff00,0xff00,0x1800,0x1800,0x1800,0x8000,0x4000,0x4000,0xc000,
	0xc000,0x0000,0x0000,0x0000,0x0000,0xf000,0xf000,0xc000,0xc000,0x8000,
	0x8000,0x8000,0xc000,0x4000,0x4000,0x6000,0x2000,0x2000,0x3000,0x1000,
	0x1000,0x3c00,0x7e00,0xe700,0xc300,0xc300,0xc300,0xc300,0xc300,0xc300,
	0xe700,0x7e00,0x3c00,0x1800,0x1800,0x1800,0x1800,0x1800,0x1800,0x1800,
	0x1800,0xf800,0xf800,0x1800,0x0800,0xff00,0xff00,0x7000,0x3800,0x1c00,
	0x0e00,0x0700,0xc300,0xc300,0xe700,0x7e00,0x3c00,0x3c00,0x7e00,0xe700,
	0xc300,0x0700,0x1e00,0x1e00,0x0700,0xc300,0xe700,0x7e00,0x3c00,0x0600,
	0x0600,0x0600,0xff00,0xff00,0xc600,0x6600,0x6600,0x3600,0x3600,0x1e00,
	0x0e00,0x3c00,0x7e00,0xe700,0x0300,0x0300,0x4700,0x7e00,0x7c00,0x3000,
	0x3000,0x3f00,0x3f00,0x3c00,0x7e00,0xe700,0xc300,0xc300,0xe700,0xfe00,
	0xdc00,0xc000,0xe700,0x7e00,0x3c00,0x3000,0x3000,0x3000,0x1800,0x1800,
	0x1800,0x0c00,0x0c00,0x0600,0x0600,0xff00,0xff00,0x3c00,0x7e00,0xe700,
	0xc300,0xe700,0x7e00,0x3c00,0x6600,0xc300,0xe700,0x7e00,0x3c00,0x3c00,
	0x7e00,0xe700,0x0300,0x3b00,0x7f00,0xe700,0xc300,0xc300,0xe700,0x7e00,
	0x3c00,0x0000,0x0000,0xc000,0xc000,0x0000,0x0000,0x0000,0x0000,0xc000,
	0xc000,0x8000,0x4000,0x4000,0xc000,0xc000,0x0000,0x0000,0x0000,0x0000,
	0xc000,0xc000,0x0000,0x0000,0x0300,0x0f00,0x3c00,0xe000,0xe000,0x3c00,
	0x0f00,0x0300,0x0000,0x0000,0x0000,0xff00,0xff00,0x0000,0x0000,0xff00,
	0xff00,0x0000,0x0000,0xc000,0xf000,0x3c00,0x0700,0x0700,0x3c00,0xf000,
	0xc000,0x1800,0x1800,0x0000,0x1800,0x1800,0x0c00,0x0e00,0x0600,0xc600,
	0xc600,0xfe00,0x7c00,0x0f80,0x38c0,0x6000,0x4660,0xc990,0x9998,0x9888,
	0x98c8,0x88c8,0xcc48,0x47d8,0x6010,0x3860,0x0fc0,0xc060,0xc060,0xffe0,
	0x7fc0,0x60c0,0x71c0,0x3180,0x3b80,0x1b00,0x1f00,0x0e00,0x0e00,0xfe00,
	0xff00,0xc380,0xc180,0xc380,0xff00,0xff00,0xc380,0xc180,0xc380,0xff00,
	0xfe00,0x1f00,0x3f80,0x71c0,0x60c0,0xe000,0xc000,0xc000,0xe000,0x60c0,
	0x71c0,0x3f80,0x1f00,0xfc00,0xff00,0xc380,0xc180,0xc1c0,0xc0c0,0xc0c0,
	0xc1c0,0xc180,0xc380,0xff00,0xfc00,0xff00,0xff00,0xc000,0xc000,0xc000,
	0xfe00,0xfe00,0xc000,0xc000,0xc000,0xff00,0xff00,0xc000,0xc000,0xc000,
	0xc000,0xc000,0xfe00,0xfe00,0xc000,0xc000,0xc000,0xff00,0xff00,0x1ec0,
	0x3fc0,0x71c0,0x60c0,0xe3c0,0xc3c0,0xc000,0xe000,0x60c0,0x71c0,0x3f80,
	0x1f00,0xc0c0,0xc0c0,0xc0c0,0xc0c0,0xc0c0,0xc0c0,0xffc0,0xffc0,0xc0c0,
	0xc0c0,0xc0c0,0xc0c0,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,
	0xc000,0xc000,0xc000,0xc000,0xc000,0x7c00,0xfe00,0xc600,0xc600,0x0600,
	0x0600,0x0600,0x0600,0x0600,0x0600,0x0600,0x0600,0xc0e0,0xc1c0,0xc380,
	0xc700,0xce00,0xfc00,0xf800,0xdc00,0xce00,0xc700,0xc380,0xc1c0,0xff00,
	0xff00,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,
	0xc000,0xc460,0xce60,0xce60,0xca60,0xdb60,0xdb60,0xd160,0xf1e0,0xf1e0,
	0xe0e0,0xe0e0,0xe0e0,0xc1c0,0xc3c0,0xc3c0,0xc6c0,0xc6c0,0xccc0,0xccc0,
	0xd8c0,0xd8c0,0xf0c0,0xf0c0,0xe0c0,0x1f00,0x3f80,0x71c0,0x60c0,0xe0e0,
	0xc060,0xc060,0xe0e0,0x60c0,0x71c0,0x3f80,0x1f00,0xc000,0xc000,0xc000,
	0xc000,0xfe00,0xff00,0xc380,0xc180,0xc180,0xc380,0xff00,0xfe00,0x0080,
	0x1fc0,0x3f80,0x73c0,0x67c0,0xe260,0xc060,0xc060,0xe0e0,0x60c0,0x71c0,
	0x3f80,0x1f00,0xc1c0,0xc1c0,0xc180,0xc180,0xc380,0xff00,0xff00,0xc380,
	0xc180,0xc180,0xff80,0xff00,0x3e00,0x7f00,0xe380,0xc180,0x0380,0x1f00,
	0x7e00,0xf000,0xc180,0xe380,0x7f00,0x3e00,0x0c00,0x0c00,0x0c00,0x0c00,
	0x0c00,0x0c00,0x0c00,0x0c00,0x0c00,0x0c00,0xffc0,0xffc0,0x3f00,0x7f80,
	0xe1c0,0xc0c0,0xc0c0,0xc0c0,0xc0c0,0xc0c0,0xc0c0,0xc0c0,0xc0c0,0xc0c0,
	0x0c00,0x0c00,0x1e00,0x1e00,0x3300,0x3300,0x3300,0x6180,0x6180,0x6180,
	0xc0c0,0xc0c0,0x1860,0x1860,0x1860,0x3cf0,0x3cf0,0x34b0,0x6798,0x6798,
	0x6318,0xc30c,0xc30c,0xc30c,0xc180,0xe380,0x6300,0x3600,0x3e00,0x1c00,
	0x1c00,0x3e00,0x3600,0x6300,0xe380,0xc180,0x0c00,0x0c00,0x0c00,0x0c00,
	0x0c00,0x1e00,0x1e00,0x3300,0x3300,0x6180,0xe1c0,0xc0c0,0xff00,0xff00,
	0xe000,0x6000,0x3000,0x3800,0x1c00,0x0c00,0x0600,0x0700,0xff00,0xff00,
	0xf000,0xf000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,
	0xc000,0xc000,0xc000,0xf000,0xf000,0x1000,0x1000,0x1000,0x3000,0x2000,
	0x2000,0x6000,0x4000,0x4000,0xc000,0x8000,0x8000,0xf000,0xf000,0x3000,
	0x3000,0x3000,0x3000,0x3000,0x3000,0x3000,0x3000,0x3000,0x3000,0x3000,
	0xf000,0xf000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
	0x0000,0xc300,0xc300,0x6600,0x6600,0x3c00,0x1800,0x1800,0xff80,0x0000,
	0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0xc000,
	0xc000,0x8000,0x4000,0x7700,0xfe00,0xc600,0xe600,0x7e00,0x0e00,0xc600,
	0xfe00,0x7c00,0xdc00,0xfe00,0xe700,0xc300,0xc300,0xc300,0xe700,0xfe00,
	0xdc00,0xc000,0xc000,0xc000,0x3c00,0x7e00,0xe700,0xc000,0xc000,0xc000,
	0xe700,0x7e00,0x3c00,0x3b00,0x7f00,0xe700,0xc300,0xc300,0xc300,0xe700,
	0x7f00,0x3f00,0x0300,0x0300,0x0300,0x3c00,0x7e00,0xe700,0xc000,0xff00,
	0xff00,0xc300,0x7e00,0x3c00,0x6000,0x6000,0x6000,0x6000,0x6000,0x6000,
	0x6000,0xf000,0xf000,0x6000,0x7000,0x3000,0x3c00,0x7e00,0xe700,0x0300,
	0x3b00,0x7f00,0xe700,0xc300,0xc300,0xc300,0xe700,0x7f00,0x3b00,0xc300,
	0xc300,0xc300,0xc300,0xc300,0xc300,0xe300,0xff00,0xde00,0xc000,0xc000,
	0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,
	0x0000,0xc000,0xc000,0xc000,0xe000,0x6000,0x6000,0x6000,0x6000,0x6000,
	0x6000,0x6000,0x6000,0x6000,0x6000,0x6000,0x0000,0x6000,0x6000,0xc700,
	0xc600,0xce00,0xec00,0xfc00,0xf800,0xdc00,0xce00,0xc700,0xc000,0xc000,
	0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,0xc000,
	0xc000,0xc000,0xc000,0xc630,0xc630,0xc630,0xc630,0xc630,0xc630,0xe730,
	0xfff0,0xdee0,0xc300,0xc300,0xc300,0xc300,0xc300,0xc300,0xe300,0xff00,
	0xde00,0x3c00,0x7e00,0xe700,0xc300,0xc300,0xc300,0xe700,0x7e00,0x3c00,
	0xc000,0xc000,0xc000,0xc000,0xdc00,0xfe00,0xe700,0xc300,0xc300,0xc300,
	0xe700,0xfe00,0xdc00,0x0300,0x0300,0x0300,0x0300,0x3b00,0x7f00,0xe700,
	0xc300,0xc300,0xc300,0xe700,0x7f00,0x3b00,0xc000,0xc000,0xc000,0xc000,
	0xc000,0xc000,0xe000,0xf800,0xd800,0x7c00,0xfe00,0xc600,0x0e00,0x7c00,
	0xf000,0xc600,0xfe00,0x7c00,0x3000,0x7000,0x6000,0x6000,0x6000,0x6000,
	0x6000,0xf000,0xf000,0x6000,0x6000,0x7b00,0xff00,0xc700,0xc300,0xc300,
	0xc300,0xc300,0xc300,0xc300,0x1800,0x1800,0x3c00,0x3c00,0x6600,0x6600,
	0x6600,0xc300,0xc300,0x1980,0x1980,0x39c0,0x3fc0,0x6f60,0x6660,0x6660,
	0xc630,0xc630,0xc600,0xc600,0x6c00,0x7c00,0x3800,0x7c00,0x6c00,0xc600,
	0xc600,0x6000,0x7000,0x1800,0x1800,0x1800,0x1c00,0x3c00,0x3600,0x3600,
	0x6600,0x6300,0xc300,0xc300,0xfe00,0xfe00,0x7000,0x3800,0x1800,0x1c00,
	0x0e00,0xfe00,0xfe00,0x3000,0x7000,0x6000,0x6000,0x6000,0x6000,0x6000,
	0xc000,0x6000,0x6000,0x6000,0x6000,0x6000,0x7000,0x3000,0xc000,0xe000,
	0x6000,0x6000,0x6000,0x6000,0x6000,0x3000,0x6000,0x6000,0x6000,0x6000,
	0x6000,0xe000,0xc000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
	0x0000,0x0000,0x8e00,0x9900,0x7100,0x0000,0x0000,0x7400,0x0000,0x0000,
	0x6d6f,0x0000,0x0000,0x6f72,0x0000,0x0000,0x696e,0x0000,0x0000,0x7269,
	0x0000,0x0000,0x6572,0x0000,0x0000,0x6563,0};
	
Fontchar	chars[256] = {	
  {0,0,0,0,0,0},{0,12,12,0,0,13},{12,0,0,0,0,0},{12,0,0,0,0,0},
  {12,0,0,0,0,0},{12,0,0,0,0,0},{12,0,0,0,0,0},{12,0,0,0,0,0},
  {12,0,0,0,0,0},{12,0,0,0,0,0},{12,0,0,0,0,0},{12,0,0,0,0,0},
  {12,0,0,0,0,0},{12,0,0,0,0,0},{12,0,0,0,0,0},{12,0,0,0,0,0},
  {12,0,0,0,0,0},{12,0,0,0,0,0},{12,27,1,0,0,26},{14,0,0,0,0,0},
  {14,0,0,0,0,0},{14,0,0,0,0,0},{14,0,0,0,0,0},{14,0,0,0,0,0},
  {14,0,0,0,0,0},{14,0,0,0,0,0},{14,0,0,0,0,0},{14,0,0,0,0,0},
  {14,0,0,0,0,0},{14,0,0,0,0,0},{14,0,0,0,0,0},{14,0,0,0,0,0},
  {14,8,1,0,0,7},{15,2,12,0,0,4},{27,5,14,0,0,7},{41,9,12,0,0,11},
  {53,7,14,0,-1,9},{67,13,12,0,0,15},{79,10,12,0,0,12},{91,2,14,0,0,4},
  {105,4,15,0,-1,6},{120,4,15,0,-1,6},{135,5,10,0,0,7},{145,8,11,0,0,10},
  {156,2,5,0,-3,4},{161,4,6,0,0,6},{167,2,2,0,0,4},{169,4,12,0,0,6},
  {181,8,12,0,0,10},{193,5,12,0,0,7},{205,8,12,0,0,10},{217,8,12,0,0,10},
  {229,8,12,0,0,10},{241,8,12,0,0,10},{253,8,12,0,0,10},{265,8,12,0,0,10},
  {277,8,12,0,0,10},{289,8,12,0,0,10},{301,2,10,0,0,4},{311,2,11,0,-3,4},
  {322,8,10,0,0,10},{332,8,9,0,0,10},{341,8,10,0,0,10},{351,7,12,0,0,9},
  {363,13,14,0,0,15},{377,11,12,0,0,13},{389,9,12,0,0,11},{401,10,12,0,0,12},
  {413,10,12,0,0,12},{425,8,12,0,0,10},{437,8,12,0,0,10},{449,10,12,0,0,12},
  {461,10,12,0,0,12},{473,2,12,0,0,4},{485,7,12,0,0,9},{497,11,12,0,0,13},
  {509,8,12,0,0,10},{521,11,12,0,0,13},{533,10,12,0,0,12},{545,11,12,0,0,13},
  {557,9,12,0,0,11},{569,11,13,0,-1,13},{582,10,12,0,0,12},{594,9,12,0,0,11},
  {606,10,12,0,0,12},{618,10,12,0,0,12},{630,10,12,0,0,12},{642,14,12,0,0,16},
  {654,9,12,0,0,11},{666,10,12,0,0,12},{678,8,12,0,0,10},{690,4,15,0,0,6},
  {705,4,12,0,0,6},{717,4,15,0,0,6},{732,8,16,0,0,10},{748,9,1,0,-2,11},
  {749,2,14,0,0,4},{763,8,9,0,0,10},{772,8,12,0,0,10},{784,8,9,0,0,10},
  {793,8,12,0,0,10},{805,8,9,0,0,10},{814,4,12,0,0,6},{826,8,13,0,-4,10},
  {839,8,12,0,0,10},{851,2,12,0,0,4},{863,3,16,0,-4,5},{879,8,12,0,0,10},
  {891,2,12,0,0,4},{903,12,9,0,0,14},{912,8,9,0,0,10},{921,8,9,0,0,10},
  {930,8,13,0,-4,10},{943,8,13,0,-4,10},{956,5,9,0,0,7},{965,7,9,0,0,9},
  {974,4,11,0,0,6},{985,8,9,0,0,10},{994,8,9,0,0,10},{1003,12,9,0,0,14},
  {1012,7,9,0,0,9},{1021,8,13,0,-4,10},{1034,7,9,0,0,9},{1043,4,15,0,-1,6},
  {1058,1,16,0,0,3},{1058,4,15,0,-1,6},{1073,8,12,0,0,10},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},
  {1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,0,0,0,0,0},{1085,33,1,0,0,32},
  {1087,33,1,0,0,32},{1089,0,0,0,0,0},{1089,33,1,0,0,32},{1091,33,1,0,0,32},
  {1093,0,0,0,0,0},{1093,33,1,0,0,32},{1095,33,1,0,0,32},{1097,33,1,0,0,32},
  {1099,0,0,0,0,0},{1099,0,0,0,0,0},{1099,0,0,0,0,0},{1099,0,0,0,0,0},
  {1099,0,0,0,0,0},{1099,0,0,0,0,0},{1099,0,0,0,0,0},{1099,0,0,0,0,0},
  {1099,0,0,0,0,0},{1099,0,0,0,0,0},{1099,0,0,0,0,0},{1099,0,0,0,0,0}
  };

/* *****************************************************************************

	Routine:	ip_font()
	Programmer:	David Cook - cookware

	Copyright:	This routine belongs to the Interplay menu manager
			system.  It was removed and slightly modified to
			allow memstat to be distributed without out the
			overhead of Interplay.  Note that the look and feel
			of this routine are the copyright of cookware, and
			are part of the Interplay menu manager.  For more
			information about Interplay, contact David Cook at
			(317) 769-5049

	Description:	This routine handles attaching the raster font to the
			font system for use with the memstat program.

	Syntax:		ip_font();

***************************************************************************** */
void	ip_font()
{
	defrasterfont(2,16,256,chars,1099,raster_f);
	font(2);
	return;
}
